/** This file is an implementation of the PLCOpen
Building Block SF_EmergencyStop, page 42, v. 1.0 */

#include <8051.h>
#include <assert.h>

#define ERROR_FAIL  0xC1
#define C_SUCCESS   0xC2

typedef unsigned char UINT8;
typedef unsigned int UINT16;

/* Define states and events. */
enum states { STATE_IDLE, STATE_INIT, STATE_WAIT_FOR_ESTOPIn1, STATE_WAIT_FOR_RESET1, STATE_RESET_ERROR1,
STATE_SAFETY_OUTP_ENABLED, STATE_WAIT_FOR_RESET2, STATE_RESET_ERROR2, STATE_WAIT_FOR_ESTOPIn2, STATE_MAX } currState;

enum events { EVENT_1, EVENT_2, MAX_EVENTS } new_event;

#define true 1
#define false 0

/** inputs */
UINT8 Activate;
UINT8 S_EStopIn;
UINT8 S_StartReset;
UINT8 S_AutoReset;
UINT8 Reset;

/** outputs */
UINT8 Ready;
UINT8 S_EStopOut;
UINT8 Error;

/** Variables */
UINT8 R_TRIGatReset;
UINT8 tmp;

UINT8 DiagCode;

void main(void){
	currState = STATE_IDLE;

	while(true){
	 	
		switch (currState){
		
			/** Rising Edge Detection */		
			R_TRIGatReset = Reset && ! tmp;
			tmp = Reset;			

			case STATE_IDLE:	
				Ready = false;
				S_EStopOut = false;
				Error = false;
				DiagCode = 0x00;

				currState = STATE_INIT;
			break; 
			case STATE_INIT:
				Ready = true;
				S_EStopOut = false;
				Error = false;
				DiagCode = 0x01;

				if(!S_StartReset) 		currState = STATE_WAIT_FOR_ESTOPIn1;
  				if(S_StartReset && !S_EStopIn) 	currState = STATE_WAIT_FOR_ESTOPIn2;
				if(S_EStopIn && S_StartReset)	currState = STATE_SAFETY_OUTP_ENABLED;
			break;
			case STATE_WAIT_FOR_ESTOPIn1:
				Ready = true;
				S_EStopOut = false;
				Error = false;
				DiagCode = 0x02;

				if(S_EStopIn)			currState = STATE_WAIT_FOR_RESET1;
			break;
			case STATE_WAIT_FOR_RESET1:
				Ready = true;
				S_EStopOut = false;
				Error = false;
				DiagCode = 0x03;

				if(Reset && ! R_TRIGatReset)	currState = STATE_RESET_ERROR1;
				if(!S_EStopIn)			currState = STATE_WAIT_FOR_ESTOPIn1;
				if(R_TRIGatReset)		currState = STATE_SAFETY_OUTP_ENABLED;
			break;
			case STATE_RESET_ERROR1:
				Ready = true;
				S_EStopOut = false;
				Error = true;
				DiagCode = 0x01;

				if(!Reset)			currState = STATE_WAIT_FOR_RESET1;
			break;
			case STATE_WAIT_FOR_ESTOPIn2:
				Ready = true;
				S_EStopOut = false;
				Error = false;
				DiagCode = 0x04;

				if(S_EStopIn && !S_AutoReset)	currState = STATE_WAIT_FOR_RESET2;
				if(S_EStopIn && S_AutoReset)	currState = STATE_SAFETY_OUTP_ENABLED;
			break;
			case STATE_WAIT_FOR_RESET2:
				Ready = true;
				S_EStopOut = false;
				Error = false;
				DiagCode = 0x05;

				if(Reset && !R_TRIGatReset)	currState = STATE_RESET_ERROR2;
				if(!S_EStopIn)			currState = STATE_WAIT_FOR_ESTOPIn2; 
				if(R_TRIGatReset || S_AutoReset)currState = STATE_SAFETY_OUTP_ENABLED;
			break;
			case STATE_RESET_ERROR2:
				Ready = true;
				S_EStopOut = false;
				Error = true;
				DiagCode = 0x02;

				if(!Reset)			currState = STATE_WAIT_FOR_RESET2;
			break;
			case STATE_SAFETY_OUTP_ENABLED:
				Ready = true;
				S_EStopOut = true;
				Error = false;
				DiagCode = 0x00;

				if(!S_EStopIn)			currState = STATE_WAIT_FOR_ESTOPIn2;
			break;
			default:
				break;
		}
	}
}